#!/bin/bash
export PATH="/usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin"

# Exit immediately if a command exits with a non-zero status
set -e

# Determine the directory where the script is located
APP_HOME="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
BREW_DIR="$APP_HOME/homebrew"
PROFILE_SCRIPT="$APP_HOME/set_homebrew_env.sh"

# Function to install Homebrew locally
install_homebrew_locally() {

  if [ -d "$BREW_DIR" ]; then
    echo "Directory $BREW_DIR already exists. Skipping Homebrew installation."
    return 0
  fi

  echo "Homebrew not found. Installing Homebrew locally in $BREW_DIR..."

  # Create the Homebrew directory
  mkdir -p "$BREW_DIR"

  # Set the HOME environment variable
  export HOME="/var/root"

  # Clone Homebrew repository into the custom directory
  git clone https://github.com/Homebrew/brew "$BREW_DIR/Homebrew"

  # Set environment variables for Homebrew
  export HOMEBREW_PREFIX="$BREW_DIR"
  export HOMEBREW_REPOSITORY="$BREW_DIR/Homebrew"
  export HOMEBREW_CELLAR="$BREW_DIR/Cellar"
  export HOMEBREW_CACHE="$BREW_DIR/Caches/Homebrew"

  # Initialize Homebrew environment for the current shell session
  eval "$("$BREW_DIR/Homebrew/bin/brew" shellenv)"

  # Set correct permissions for Homebrew directories
  chmod -R u+w "$BREW_DIR"
  chown -R "$(whoami)" "$BREW_DIR"

  # Create a profile script to set Homebrew environment variables
  cat <<EOL > "$PROFILE_SCRIPT"
# Homebrew Environment Variables
export HOME="/var/root"
export HOMEBREW_PREFIX="$BREW_DIR"
export HOMEBREW_REPOSITORY="$BREW_DIR/Homebrew"
export HOMEBREW_CELLAR="$BREW_DIR/Cellar"
export HOMEBREW_CACHE="$BREW_DIR/Caches/Homebrew"
eval "\$("$BREW_DIR/Homebrew/bin/brew" shellenv)"
EOL

  # Make the profile script executable
  chmod +x "$PROFILE_SCRIPT"

  echo "Homebrew has been installed locally in $BREW_DIR."
  echo "Created Homebrew environment profile at $PROFILE_SCRIPT."
}

# Function to set up Homebrew environment
setup_homebrew_env() {
  echo "Setting up Homebrew environment..."

  # Source the profile script to set environment variables
  if [ -f "$PROFILE_SCRIPT" ]; then
    source "$PROFILE_SCRIPT"
    echo "Sourced Homebrew environment from $PROFILE_SCRIPT."
  else
    echo "Profile script $PROFILE_SCRIPT not found. Exiting."
    exit 1
  fi
}

# Check if Homebrew is already installed
if command -v brew &>/dev/null; then
  echo "Homebrew is already installed. Using existing Homebrew."
  BREW_CMD="brew"
else
  # Install Homebrew locally
  install_homebrew_locally
  BREW_CMD="$BREW_DIR/Homebrew/bin/brew"
fi

# If Homebrew was installed locally, set up the environment
if [ "$BREW_CMD" != "brew" ]; then
  setup_homebrew_env
fi

# Update Homebrew to ensure the latest formulas are available
echo "Updating Homebrew..."
"$BREW_CMD" update || {
  echo "Failed to update Homebrew."
  exit 1
}

# Function to install FFmpeg@7 if not already installed
install_ffmpeg() {
  echo "Checking if FFmpeg@7 is installed..."

  if "$BREW_CMD" list --versions ffmpeg@7 &>/dev/null; then
    echo "FFmpeg@7 is already installed."
  else
    echo "FFmpeg@7 is not installed. Installing FFmpeg@7..."
    "$BREW_CMD" install ffmpeg@7 --force-bottle || {
      echo "Failed to install FFmpeg@7."
      exit 1
    }
  fi

  echo "Linking FFmpeg@7 to make it accessible..."
  "$BREW_CMD" link ffmpeg@7 --force --overwrite || {
    echo "Failed to link FFmpeg@7."
    exit 1
  }

  echo "Verifying FFmpeg installation..."
  if command -v ffmpeg &>/dev/null; then
    echo "FFmpeg@7 installed and linked successfully."
    ffmpeg -version
  else
    echo "FFmpeg installation verification failed."
    exit 1
  fi
}

# Install FFmpeg@7
install_ffmpeg

# Function to unlink older versions of FFmpeg if present
unlink_older_ffmpeg() {
  OLD_VERSION="$1"
  echo "Checking for older FFmpeg version: $OLD_VERSION..."

  if "$BREW_CMD" list --versions "$OLD_VERSION" &>/dev/null; then
    echo "Found $OLD_VERSION. Unlinking..."
    "$BREW_CMD" unlink "$OLD_VERSION" || {
      echo "Failed to unlink $OLD_VERSION."
      exit 1
    }
    echo "$OLD_VERSION has been unlinked."
  else
    echo "$OLD_VERSION is not installed. No action needed."
  fi
}

# Example: Unlink FFmpeg@6 if installed
unlink_older_ffmpeg "ffmpeg@6"

echo "Installation script completed successfully."

# Instructions for updating PATH in the service environment
echo "Ensure that your service sources the Homebrew environment profile to access FFmpeg."
echo "For example, in your service configuration, include:"
echo "source \"$PROFILE_SCRIPT\""
echo "or add '$BREW_DIR/Homebrew/bin' to the PATH environment variable."
